#include <windows.h>
#include <atlbase.h>

CComModule _Module;

#include <atlcom.h>
#include <atlwin.h>
#include "webview.h"
#include "HTMLView.h"
#include "exdisp.h"

static const CLSID clsid = {0x698E3FCA, 0x70C3, 0x11D0, {0x81, 0xE8, 0x00, 0xA0, 0xC9, 0x0A, 0xD2, 0x0A } };

IBrowser* pBrowser = NULL;

int WINAPI WinMain(
			HINSTANCE hInstance, 
			HINSTANCE hPrevInstance, 
			LPTSTR lpCmdLine,
			int nCmdShow)
{
	MSG			msg;
	WNDCLASS	wndClass;

	ghInst = hInstance;

	/* Register the main window class */
	wndClass.style         = 0;
	wndClass.lpfnWndProc   = MainWndProc;
	wndClass.cbClsExtra    = 0;
	wndClass.cbWndExtra    = 0;
	wndClass.hInstance     = hInstance;
	wndClass.hIcon         = NULL;
	wndClass.hCursor       = NULL;
	wndClass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndClass.lpszMenuName  = NULL;
	wndClass.lpszClassName = pszAppName;

    ::RegisterClass(&wndClass);

	// get work area size
	::SystemParametersInfo(SPI_GETWORKAREA, sizeof(RECT), &screenRECT, 0);

    hwndMain = ::CreateWindowEx(
        WS_EX_NODRAG,
        pszAppName,
        TEXT("HTMLView"),
        WS_VISIBLE | WS_BORDER,
        0,0,
        (screenRECT.right - screenRECT.left),
        (screenRECT.bottom - screenRECT.top),
        NULL,
        NULL,
        hInstance,
        NULL
        );

	// main message pump
    while (::GetMessage(&msg, NULL, 0, 0) == TRUE)
	{
        ::TranslateMessage(&msg);
        ::DispatchMessage(&msg); 
	}
	return (msg.wParam);
}

BOOL CreateBrowser(HWND hwnd)
{
    HRESULT		hr;
	IUnknown*	pUnk = NULL;
    CAxWindow*	paxWindow = NULL;
    CComPtr<IAxWinHostWindow> spHost;

    paxWindow = new CAxWindow();
    paxWindow->Create(hwnd, 
					  &screenRECT, 
					  _T("{698E3FCA-70C3-11D0-81E8-00A0C90AD20A}"), 
					  (WS_VISIBLE));
        
    paxWindow->QueryHost(&spHost);

	hr = paxWindow->QueryControl(
			IID_IBrowser,
			(void**)&pBrowser);


    pBrowserEvents = NULL;
    pEventSink = new CWebBrowserEvents();

	IConnectionPointContainer* pCPC = NULL;
	IConnectionPoint* pCP = NULL;
	IEnumConnectionPoints* pECP = NULL;

	hr = pBrowser->QueryInterface(
        IID_IConnectionPointContainer,
		(void**)&pCPC);

	if (SUCCEEDED(hr))
	{
		hr = pCPC->FindConnectionPoint(
			DIID__IBrowserEvents,
			(IConnectionPoint**)&pCP
			);
		if (FAILED(hr))
		{
			pCPC->EnumConnectionPoints(
				(IEnumConnectionPoints**)&pECP
				);
			ULONG nCnt = 1;
			IUnknown* pUnk = reinterpret_cast<IUnknown*>(pEventSink);
			while(nCnt)
			{   //We know that there is only one implemented connection point here
				pECP->Next(
					1,
					(IConnectionPoint**)&pBrowserEvents,
					&nCnt
					);
				hr = pBrowserEvents->Advise(
					pUnk,
					&dwCookie
					);
			}   //End of while(nCnt) loop
		}		//End of if (FAILED(hr)) statement
		pBrowser->Release();  //Release the IID_IConnectionPointContainer interface
	}	//End of if(SUCCEEDED(hr)) statment (IID_IConnectionPointContainer QueryInterface)
	else
	{
		//Could not find IConnectionPointContainer Interface
		return (FALSE);
	}
	pCPC->Release();

    if (SUCCEEDED(hr))
    {
		pBrowser->Navigate(pszStartupURL, 0, NULL, NULL);
    }
    else
    {
        ::MessageBox(hwnd, TEXT("QueryControl Failed"), NULL, MB_OK);
    }
	return (TRUE);
}


LRESULT CALLBACK MainWndProc(
    HWND hwnd,
    UINT message,
    WPARAM wParam,
    LPARAM lParam
    )
{
	switch(message)
	{
	case WM_CREATE:
	    ::CoInitializeEx(NULL, COINIT_MULTITHREADED);
        AtlAxWinInit();
        CreateBrowser(hwnd);
        break;

    case WM_DESTROY:
        pBrowserEvents->Unadvise(
            dwCookie
            );
        ::CoUninitialize();
        ::DestroyWindow(hwnd);
        ::PostQuitMessage(0);
        break;

	default:
		return (DefWindowProc(hwnd, message, wParam, lParam));
	}

    return (0);
}

